@@ -8,7 +8,7 @@ import unittest
 import mock
 from mock import patch
 from nose import SkipTest
-from nose.tools import eq_
+from nose.tools import eq_, ok_, assert_not_equal
 from nose.tools import raises
 
 from kazoo.testing import KazooTestCase
@@ -179,34 +179,6 @@ class TestAuthentication(KazooTestCase):
             client.stop()
             client.close()
 
-    def test_connect_sasl_auth(self):
-        from kazoo.security import make_acl
-
-        if TRAVIS_ZK_VERSION:
-            version = TRAVIS_ZK_VERSION
-        else:
-            version = self.client.server_version()
-        if not version or version < (3, 4):
-            raise SkipTest("Must use Zookeeper 3.4 or above")
-
-        username = "jaasuser"
-        password = "jaas_password"
-        sasl_auth = "%s:%s" % (username, password)
-
-        acl = make_acl('sasl', credential=username, all=True)
-
-        client = self._get_client(auth_data=[('sasl', sasl_auth)])
-        client.start()
-        try:
-            client.create('/1', acl=(acl,))
-            # give ZK a chance to copy data to other node
-            time.sleep(0.1)
-            self.assertRaises(NoAuthError, self.client.get, "/1")
-        finally:
-            client.delete('/1')
-            client.stop()
-            client.close()
-
     def test_unicode_auth(self):
         username = u("xe4/\hm")
         password = u("/\xe4hm")
@@ -245,16 +217,6 @@ class TestAuthentication(KazooTestCase):
         self.assertRaises(TypeError, client.add_auth,
                           None, ('user', 'pass'))
 
-    def test_invalid_sasl_auth(self):
-        if TRAVIS_ZK_VERSION:
-            version = TRAVIS_ZK_VERSION
-        else:
-            version = self.client.server_version()
-        if not version or version < (3, 4):
-            raise SkipTest("Must use Zookeeper 3.4 or above")
-        client = self._get_client(auth_data=[('sasl', 'baduser:badpassword')])
-        self.assertRaises(ConnectionLoss, client.start)
-
     def test_async_auth(self):
         client = self._get_client()
         client.start()
@@ -530,9 +492,6 @@ class TestClient(KazooTestCase):
         self.assertRaises(AuthFailedError, client.create,
                           '/closedpath', b'bar')
 
-        client._state = KeeperState.CONNECTING
-        self.assertRaises(SessionExpiredError, client.create,
-                          '/closedpath', b'bar')
         client.stop()
         client.close()
 
@@ -713,6 +672,19 @@ class TestClient(KazooTestCase):
         path = client.create("/1")
         self.assertRaises(NodeExistsError, client.create, path)
 
+    def test_create_stat(self):
+        if TRAVIS_ZK_VERSION:
+            version = TRAVIS_ZK_VERSION
+        else:
+            version = self.client.server_version()
+        if not version or version < (3, 5):
+            raise SkipTest("Must use Zookeeper 3.5 or above")
+        client = self.client
+        path, stat1 = client.create("/1", b"bytes", include_data=True)
+        data, stat2 = client.get("/1")
+        eq_(data, b"bytes")
+        eq_(stat1, stat2)
+
     def test_create_get_set(self):
         nodepath = "/" + uuid.uuid4().hex
 
@@ -825,10 +797,13 @@ class TestClient(KazooTestCase):
 
     def test_get_acls(self):
         from kazoo.security import make_digest_acl
-        acl = make_digest_acl('user', 'pass', all=True)
+        user = 'user'
+        passw = 'pass'
+        acl = make_digest_acl(user, passw, all=True)
         client = self.client
         try:
             client.create('/a', acl=[acl])
+            client.add_auth('digest', '{}:{}'.format(user, passw))
             self.assertTrue(acl in client.get_acls('/a')[0])
         finally:
             client.delete('/a')
@@ -839,11 +814,14 @@ class TestClient(KazooTestCase):
 
     def test_set_acls(self):
         from kazoo.security import make_digest_acl
-        acl = make_digest_acl('user', 'pass', all=True)
+        user = 'user'
+        passw = 'pass'
+        acl = make_digest_acl(user, passw, all=True)
         client = self.client
         client.create('/a')
         try:
             client.set_acls('/a', [acl])
+            client.add_auth('digest', '{}:{}'.format(user, passw))
             self.assertTrue(acl in client.get_acls('/a')[0])
         finally:
             client.delete('/a')
@@ -1001,6 +979,107 @@ class TestClient(KazooTestCase):
         finally:
             self.cluster[0].run()
 
+    # utility for test_request_queuing*
+    def _make_request_queuing_client(self):
+        from kazoo.client import KazooClient
+        server = self.cluster[0]
+        handler = self._makeOne()
+        # create a client with only one server in its list, and
+        # infinite retries
+        client = KazooClient(
+            hosts=server.address + self.client.chroot,
+            handler=handler,
+            connection_retry=dict(
+                max_tries=-1,
+                delay=0.1,
+                backoff=1,
+                max_jitter=0.0,
+                sleep_func=handler.sleep_func
+            )
+        )
+
+        return client, server
+
+    # utility for test_request_queuing*
+    def _request_queuing_common(self, client, server, path, expire_session):
+        ev_suspended = client.handler.event_object()
+        ev_connected = client.handler.event_object()
+
+        def listener(state):
+            if state == KazooState.SUSPENDED:
+                ev_suspended.set()
+            elif state == KazooState.CONNECTED:
+                ev_connected.set()
+        client.add_listener(listener)
+
+        # wait for the client to connect
+        client.start()
+
+        try:
+            # force the client to suspend
+            server.stop()
+
+            ev_suspended.wait(5)
+            ok_(ev_suspended.is_set())
+            ev_connected.clear()
+
+            # submit a request, expecting it to be queued
+            result = client.create_async(path)
+            assert_not_equal(len(client._queue), 0)
+            eq_(result.ready(), False)
+            eq_(client.state, KazooState.SUSPENDED)
+
+            # optionally cause a SessionExpiredError to occur by
+            # mangling the first byte of the session password.
+            if expire_session:
+                b0 = b'\x00'
+                if client._session_passwd[0] == 0:
+                    b0 = b'\xff'
+                client._session_passwd = b0 + client._session_passwd[1:]
+        finally:
+            server.run()
+
+        # wait for the client to reconnect (either with a recovered
+        # session, or with a new one if expire_session was set)
+        ev_connected.wait(5)
+        ok_(ev_connected.is_set())
+
+        return result
+
+    def test_request_queuing_session_recovered(self):
+        path = "/" + uuid.uuid4().hex
+        client, server = self._make_request_queuing_client()
+
+        try:
+            result = self._request_queuing_common(
+                client=client,
+                server=server,
+                path=path,
+                expire_session=False
+            )
+
+            eq_(result.get(), path)
+            assert_not_equal(client.exists(path), None)
+        finally:
+            client.stop()
+
+    def test_request_queuing_session_expired(self):
+        path = "/" + uuid.uuid4().hex
+        client, server = self._make_request_queuing_client()
+
+        try:
+            result = self._request_queuing_common(
+                client=client,
+                server=server,
+                path=path,
+                expire_session=True
+            )
+
+            eq_(len(client._queue), 0)
+            self.assertRaises(SessionExpiredError, result.get)
+        finally:
+            client.stop()
+
 
 dummy_dict = {
     'aversion': 1, 'ctime': 0, 'cversion': 1,
@@ -1031,8 +1110,8 @@ class TestClientTransactions(KazooTestCase):
         t.create('/fred', ephemeral=True)
         t.create('/smith', sequence=True)
         results = t.commit()
-        eq_(results[0], '/freddy')
         eq_(len(results), 3)
+        eq_(results[0], '/freddy')
         self.assertTrue(results[2].startswith('/smith0'))
 
     def test_bad_creates(self):
@@ -1266,13 +1345,13 @@ class TestReconfig(KazooTestCase):
         joining = 'server.100=0.0.0.0:%d:%d:observer;0.0.0.0:%d' % (
             port1, port2, port3)
         data, _ = client.reconfig(joining=joining,
-                                       leaving=None,
-                                       new_members=None)
+                                  leaving=None,
+                                  new_members=None)
         self.assertIn(joining.encode('utf8'), data)
 
         data, _ = client.reconfig(joining=None,
-                                       leaving='100',
-                                       new_members=None)
+                                  leaving='100',
+                                  new_members=None)
         self.assertNotIn(joining.encode('utf8'), data)
 
         # try to add it again, but a config number in the future
