@@ -1,12 +1,12 @@
 import collections
 import threading
-import time
 import uuid
 
 from nose.tools import eq_, ok_
 
 from kazoo.exceptions import CancelledError
 from kazoo.exceptions import LockTimeout
+from kazoo.exceptions import NoNodeError
 from kazoo.testing import KazooTestCase
 from kazoo.tests import util as test_util
 
@@ -319,7 +319,6 @@ class KazooLockTests(KazooTestCase):
         sleep_func = self.client.handler.sleep_func
         lock = self.client.Lock(self.lockpath, "one")
         acquires = collections.deque()
-        chain = collections.deque()
         differences = collections.deque()
         barrier = SleepBarrier(self.thread_count, sleep_func)
 
@@ -357,6 +356,18 @@ class KazooLockTests(KazooTestCase):
         lock.acquire()
         lock.release()
 
+    def test_lock_ephemeral(self):
+        client1 = self._get_client()
+        client1.start()
+        lock = client1.Lock(self.lockpath, "ephemeral")
+        lock.acquire(ephemeral=False)
+        znode = self.lockpath + '/' + lock.node
+        client1.stop()
+        try:
+            self.client.get(znode)
+        except NoNodeError:
+            self.fail("NoNodeError raised unexpectedly!")
+
     def test_lock_timeout(self):
         timeout = 3
         e = self.make_event()
@@ -398,6 +409,50 @@ class KazooLockTests(KazooTestCase):
             t.join()
             client2.stop()
 
+    def test_read_lock(self):
+        # Test that we can obtain a read lock
+        lock = self.client.ReadLock(self.lockpath, "reader one")
+        gotten = lock.acquire(blocking=False)
+        self.assertTrue(gotten)
+        self.assertTrue(lock.is_acquired)
+        # and that it's still not reentrant.
+        gotten = lock.acquire(blocking=False)
+        self.assertFalse(gotten)
+
+        # Test that a second client we can share the same read lock
+        client2 = self._get_client()
+        client2.start()
+        lock2 = client2.ReadLock(self.lockpath, "reader two")
+        gotten = lock2.acquire(blocking=False)
+        self.assertTrue(gotten)
+        self.assertTrue(lock2.is_acquired)
+        gotten = lock2.acquire(blocking=False)
+        self.assertFalse(gotten)
+
+        # Test that a writer is unable to share it
+        client3 = self._get_client()
+        client3.start()
+        lock3 = client3.WriteLock(self.lockpath, "writer")
+        gotten = lock3.acquire(blocking=False)
+        self.assertFalse(gotten)
+
+    def test_write_lock(self):
+        # Test that we can obtain a write lock
+        lock = self.client.WriteLock(self.lockpath, "writer")
+        gotten = lock.acquire(blocking=False)
+        self.assertTrue(gotten)
+        self.assertTrue(lock.is_acquired)
+        gotten = lock.acquire(blocking=False)
+        self.assertFalse(gotten)
+
+        # Test that we are unable to obtain a read lock while the
+        # write lock is held.
+        client2 = self._get_client()
+        client2.start()
+        lock2 = client2.ReadLock(self.lockpath, "reader")
+        gotten = lock2.acquire(blocking=False)
+        self.assertFalse(gotten)
+
 
 class TestSemaphore(KazooTestCase):
 
